@@ -25,8 +25,7 @@
 
 #include <glib.h>
 #include <glib/gi18n.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtkrc.h>
+#include <gtk/gtk.h>
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -112,6 +111,7 @@ GCRY_THREAD_OPTION_PTHREAD_IMPL;
 #include "matcher.h"
 #include "tags.h"
 #include "hooks.h"
+#include "menu.h"
 
 #ifdef HAVE_LIBETPAN
 #include "imap-thread.h"
@@ -139,7 +139,6 @@ GCRY_THREAD_OPTION_PTHREAD_IMPL;
 #include <hildon-widgets/hildon-banner.h>
 #include <hildon-widgets/hildon-program.h>
 #endif
-#include <gtk/gtkmain.h>
 #include <libosso.h>
 #include <libgnomevfs/gnome-vfs-volume.h>
 #include <libgnomevfs/gnome-vfs-volume-monitor.h>
@@ -322,7 +321,7 @@ static void startup_notification_complete(gboolean with_window)
 		 * if we have been launched from a menu.
 		 * We have to display a window, so let it be very little */
 		hack = gtk_window_new(GTK_WINDOW_POPUP);
-		gtk_widget_set_uposition(hack, 0, 0);
+		gtk_window_move(GTK_WINDOW(hack), 0, 0);
 		gtk_widget_set_size_request(hack, 1, 1);
 		gtk_widget_show(hack);
 	}
@@ -420,17 +419,16 @@ static gboolean migrate_old_config(const gchar *old_cfg_dir, const gchar *new_cf
 	gint r = 0;
 	GtkWidget *window = NULL;
 	GtkWidget *keep_backup_chk;
-	GtkTooltips *tips = gtk_tooltips_new();
+	CLAWS_TIP_DECL();
 	gboolean backup = TRUE;
 
 	keep_backup_chk = gtk_check_button_new_with_label (_("Keep old configuration"));
 	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(keep_backup_chk), TRUE);
-	gtk_tooltips_set_tip(GTK_TOOLTIPS(tips), keep_backup_chk,
+	CLAWS_SET_TIP(keep_backup_chk,
 			     _("Keeping a backup will allow you to go back to an "
 			       "older version, but may take a while if you have "
 			       "cached IMAP or News data, and will take some extra "
-			       "room on your disk."),
-			     NULL);
+			       "room on your disk."));
 
 	g_signal_connect(G_OBJECT(keep_backup_chk), "toggled", 
 			G_CALLBACK(chk_update_val), &backup);
@@ -804,7 +802,7 @@ static void win32_open_log(void)
 		if (rename_force("claws-win32.log", "claws-win32.log.bak") < 0)
 			FILE_OP_ERROR("claws-win32.log", "rename");
 	}
-	win32_debug_fp = fopen("claws-win32.log", "w");
+	win32_debug_fp = g_fopen("claws-win32.log", "w");
 	if (win32_debug_fp)
 	{
 		g_set_print_handler(win32_print_stdout);
@@ -838,13 +836,22 @@ static void main_dump_features_list(gboolean show_debug_only)
 		return;
 
 	if (show_debug_only)
-		debug_print("GTK+ %d.%d.%d / GLib %d.%d.%d\n",
+		debug_print("runtime GTK+ %d.%d.%d / GLib %d.%d.%d\n",
 			   gtk_major_version, gtk_minor_version, gtk_micro_version,
 			   glib_major_version, glib_minor_version, glib_micro_version);
 	else
-		g_print("GTK+ %d.%d.%d / GLib %d.%d.%d\n",
+		g_print("runtime GTK+ %d.%d.%d / GLib %d.%d.%d\n",
 			   gtk_major_version, gtk_minor_version, gtk_micro_version,
 			   glib_major_version, glib_minor_version, glib_micro_version);
+	if (show_debug_only)
+		debug_print("buildtime GTK+ %d.%d.%d / GLib %d.%d.%d\n",
+			   GTK_MAJOR_VERSION, GTK_MINOR_VERSION, GTK_MICRO_VERSION,
+			   GLIB_MAJOR_VERSION, GLIB_MINOR_VERSION, GLIB_MICRO_VERSION);
+	else
+		g_print("buildtime GTK+ %d.%d.%d / GLib %d.%d.%d\n",
+			   GTK_MAJOR_VERSION, GTK_MINOR_VERSION, GTK_MICRO_VERSION,
+			   GLIB_MAJOR_VERSION, GLIB_MINOR_VERSION, GLIB_MICRO_VERSION);
+	
 	if (show_debug_only)
 		debug_print("Compiled-in features:\n");
 	else
@@ -855,7 +862,7 @@ static void main_dump_features_list(gboolean show_debug_only)
 	else
 		g_print(" compface\n");
 #endif
-#if USE_ASPELL
+#if USE_ENCHANT
 	if (show_debug_only)
 		debug_print(" aspell\n");
 	else
@@ -967,7 +974,7 @@ static void dbus_update(FolderItem *removed_item)
 			"claws-mail", G_TYPE_INVALID, G_TYPE_INVALID);
 	}
 	if (error) {
-		debug_print(error->message);
+		debug_print("%s\n", error->message);
 		g_error_free(error);
 	}
 }
@@ -1044,7 +1051,10 @@ int main(int argc, char *argv[])
 	gint num_folder_class = 0;
 	gboolean asked_for_migration = FALSE;
 	gboolean start_done = TRUE;
-	
+	GtkUIManager *gui_manager = NULL;
+	GSList *plug_list = NULL;
+	gboolean never_ran = FALSE;
+
 	START_TIMING("startup");
 
 	sc_starting = TRUE;
@@ -1112,6 +1122,13 @@ int main(int argc, char *argv[])
 	gtk_set_locale();
 	gtk_init(&argc, &argv);
 
+#ifdef G_OS_WIN32
+	gtk_settings_set_string_property(gtk_settings_get_default(),
+			"gtk-theme-name",
+			"MS-Windows",
+			"XProperty");
+#endif
+
 #ifdef HAVE_NETWORKMANAGER_SUPPORT
 	went_offline_nm = FALSE;
 	nm_proxy = NULL;
@@ -1150,9 +1167,12 @@ int main(int argc, char *argv[])
 	hildon_program = HILDON_PROGRAM(hildon_program_get_instance());
 	static_osso_context = osso_context;
 #endif	
-	gdk_rgb_init();
 	gtk_widget_set_default_colormap(gdk_rgb_get_colormap());
-	gtk_widget_set_default_visual(gdk_rgb_get_visual());
+
+	gui_manager = gtkut_create_ui_manager();
+
+	/* Create container for all the menus we will be adding */
+	MENUITEM_ADDUI("/", "Menus", NULL, GTK_UI_MANAGER_MENUBAR);
 
 	if (!g_thread_supported()) {
 		g_error(_("g_thread is not supported by glib.\n"));
@@ -1302,7 +1322,7 @@ int main(int argc, char *argv[])
 	prefs_receive_init();
 	prefs_send_init();
 	tags_read_tags();
-#ifdef USE_ASPELL
+#ifdef USE_ENCHANT
 	gtkaspell_checkers_init();
 	prefs_spelling_init();
 #endif
@@ -1359,12 +1379,12 @@ int main(int argc, char *argv[])
 	manage_window_focus_in(mainwin->window, NULL, NULL);
 	folderview = mainwin->folderview;
 
-	gtk_clist_freeze(GTK_CLIST(mainwin->folderview->ctree));
+	gtk_cmclist_freeze(GTK_CMCLIST(mainwin->folderview->ctree));
 	folder_item_update_freeze();
 
 	/* register the callback of unix domain socket input */
 #ifdef G_OS_UNIX
-	lock_socket_tag = gdk_input_add(lock_socket,
+	lock_socket_tag = claws_input_add(lock_socket,
 					GDK_INPUT_READ | GDK_INPUT_EXCEPTION,
 					lock_socket_input_cb,
 					mainwin);
@@ -1397,6 +1417,7 @@ int main(int argc, char *argv[])
 		}
 		main_window_reflect_prefs_all_now();
 		folder_write_list();
+		never_ran = TRUE;
 	}
 
 	if (!account_get_list()) {
@@ -1416,6 +1437,7 @@ int main(int argc, char *argv[])
 #endif
 			exit(1);
 		}
+		never_ran = TRUE;
 	}
 
 	
@@ -1493,16 +1515,38 @@ int main(int argc, char *argv[])
 		prefs_matcher_read_config();
 	}
 	
-	if (plugin_get_unloaded_list() != NULL) {
+	if ((plug_list = plugin_get_unloaded_list()) != NULL) {
+		GSList *cur;
+		gchar *list = NULL;
+		gint num_plugins = 0;
+		for (cur = plug_list; cur; cur = cur->next) {
+			Plugin *plugin = (Plugin *)cur->data;
+			gchar *tmp = g_strdup_printf("%s\n%s",
+				list? list:"",
+				plugin_get_name(plugin));
+			g_free(list);
+			list = tmp;
+			num_plugins++;
+		}
 		main_window_cursor_normal(mainwin);
-		alertpanel_warning(_("Some plugin(s) failed to load. "
+		alertpanel_warning(ngettext(
+				     "The following plugin failed to load. "
 				     "Check the Plugins configuration "
-				     "for more information."));
+				     "for more information:\n%s",
+				     "The following plugins failed to load. "
+				     "Check the Plugins configuration "
+				     "for more information:\n%s", 
+				     num_plugins), 
+				     list);
 		main_window_cursor_wait(mainwin);
+		g_free(list);
+		g_slist_free(plug_list);
 	}
 
- 	plugin_load_standard_plugins ();
-      
+	if (never_ran) {
+		prefs_common_write_config();
+	 	plugin_load_standard_plugins ();
+	}
 	/* if not crashed, show window now */
 	if (!claws_crashed()) {
 		/* apart if something told not to show */
@@ -1573,7 +1617,7 @@ int main(int argc, char *argv[])
 #endif
 
 	folder_item_update_thaw();
-	gtk_clist_thaw(GTK_CLIST(mainwin->folderview->ctree));
+	gtk_cmclist_thaw(GTK_CMCLIST(mainwin->folderview->ctree));
 	main_window_cursor_normal(mainwin);
 
 	if (!cmd.target && prefs_common.goto_last_folder_on_startup &&
@@ -1680,7 +1724,7 @@ static void exit_claws(MainWindow *mainwin)
 	if(mainwin->folderview->opened) {
 		FolderItem *item;
 
-		item = gtk_ctree_node_get_row_data(GTK_CTREE(mainwin->folderview->ctree), mainwin->folderview->opened);
+		item = gtk_cmctree_node_get_row_data(GTK_CMCTREE(mainwin->folderview->ctree), mainwin->folderview->opened);
 		summary_save_prefs_to_folderitem(mainwin->folderview->summaryview, item);
 		prefs_common.last_opened_folder = folder_item_get_identifier(item);
 	}
@@ -1747,7 +1791,7 @@ static void exit_claws(MainWindow *mainwin)
 	prefs_logging_done();
 	prefs_send_done();
 	tags_write_tags();
-#ifdef USE_ASPELL       
+#ifdef USE_ENCHANT       
 	prefs_spelling_done();
 	gtkaspell_checkers_quit();
 #endif
@@ -2224,7 +2268,7 @@ static gint lock_socket_remove(void)
 	}
 
 	if (lock_socket_tag > 0) {
-		gdk_input_remove(lock_socket_tag);
+		g_source_remove(lock_socket_tag);
 	}
 	fd_close(lock_socket);
 	filename = claws_get_socket_name();
