@@ -26,7 +26,6 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
-#include <gtk/gtkoptionmenu.h>
 #include <gdk/gdkkeysyms.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -100,29 +99,29 @@ static void prefs_filtering_set_list	(void);
 static gboolean prefs_filtering_search_func_cb (GtkTreeModel *model, gint column, 
 						const gchar *key, GtkTreeIter *iter, 
 						gpointer search_data);
-static void prefs_filtering_register_cb	(void);
-static void prefs_filtering_substitute_cb	(void);
-static void prefs_filtering_delete_cb	(void);
-static void prefs_filtering_delete_all_cb(void);
-static void prefs_filtering_clear_cb(void);
-static void prefs_filtering_duplicate_cb(void);
-static void prefs_filtering_top		(void);
-static void prefs_filtering_page_up(void);
-static void prefs_filtering_up		(void);
-static void prefs_filtering_down	(void);
-static void prefs_filtering_page_down(void);
-static void prefs_filtering_bottom	(void);
+static void prefs_filtering_register_cb	(gpointer action, gpointer data);
+static void prefs_filtering_substitute_cb	(gpointer action, gpointer data);
+static void prefs_filtering_delete_cb	(gpointer action, gpointer data);
+static void prefs_filtering_delete_all_cb(gpointer action, gpointer data);
+static void prefs_filtering_clear_cb(gpointer action, gpointer data);
+static void prefs_filtering_duplicate_cb(gpointer action, gpointer data);
+static void prefs_filtering_top		(gpointer action, gpointer data);
+static void prefs_filtering_page_up	(gpointer action, gpointer data);
+static void prefs_filtering_up		(gpointer action, gpointer data);
+static void prefs_filtering_down	(gpointer action, gpointer data);
+static void prefs_filtering_page_down	(gpointer action, gpointer data);
+static void prefs_filtering_bottom	(gpointer action, gpointer data);
 static gint prefs_filtering_deleted	(GtkWidget	*widget,
 					 GdkEventAny	*event,
 					 gpointer	 data);
 static gboolean prefs_filtering_key_pressed(GtkWidget	*widget,
 					 GdkEventKey	*event,
 					 gpointer	 data);
-static void prefs_filtering_cancel	(void);
-static void prefs_filtering_ok		(void);
+static void prefs_filtering_cancel	(gpointer action, gpointer data);
+static void prefs_filtering_ok		(gpointer action, gpointer data);
 
-static void prefs_filtering_condition_define	(void);
-static void prefs_filtering_action_define(void);
+static void prefs_filtering_condition_define	(gpointer action, gpointer data);
+static void prefs_filtering_action_define	(gpointer action, gpointer data);
 static gint prefs_filtering_list_view_set_row	(gint row, FilteringProp * prop);
 					  
 static void prefs_filtering_reset_dialog	(void);
@@ -329,12 +328,10 @@ static void prefs_filtering_create(void)
 	GtkWidget *bottom_btn;
 	GtkWidget *table;
 	static GdkGeometry geometry;
-	GtkTooltips *tooltips;
+	CLAWS_TIP_DECL();
 
 	debug_print("Creating filtering configuration window...\n");
 
-	tooltips = gtk_tooltips_new();
-
 	window = gtkut_window_new(GTK_WINDOW_TOPLEVEL, "prefs_filtering");
 	gtk_container_set_border_width (GTK_CONTAINER (window), 8);
 	gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
@@ -469,8 +466,8 @@ static void prefs_filtering_create(void)
 	gtk_box_pack_start (GTK_BOX (btn_hbox), reg_btn, FALSE, TRUE, 0);
 	g_signal_connect(G_OBJECT (reg_btn), "clicked",
 			 G_CALLBACK(prefs_filtering_register_cb), NULL);
-	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), reg_btn,
-			_("Append the new rule above to the list"), NULL);
+	CLAWS_SET_TIP(reg_btn,
+			_("Append the new rule above to the list"));
 
 	subst_btn = gtkut_get_replace_btn (_("Replace"));
 	gtk_widget_show (subst_btn);
@@ -478,24 +475,24 @@ static void prefs_filtering_create(void)
 	g_signal_connect(G_OBJECT (subst_btn), "clicked",
 			 G_CALLBACK(prefs_filtering_substitute_cb),
 			 NULL);
-	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), subst_btn,
-			_("Replace the selected rule in list with the rule above"), NULL);
+	CLAWS_SET_TIP(subst_btn,
+			_("Replace the selected rule in list with the rule above"));
 
 	del_btn = gtk_button_new_from_stock (GTK_STOCK_DELETE);
 	gtk_widget_show (del_btn);
 	gtk_box_pack_start (GTK_BOX (btn_hbox), del_btn, FALSE, TRUE, 0);
 	g_signal_connect(G_OBJECT (del_btn), "clicked",
 			G_CALLBACK(prefs_filtering_delete_cb), NULL);
-	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), del_btn,
-			_("Delete the selected rule from the list"), NULL);
+	CLAWS_SET_TIP(del_btn,
+			_("Delete the selected rule from the list"));
 
 	clear_btn = gtk_button_new_from_stock (GTK_STOCK_CLEAR);
 	gtk_widget_show (clear_btn);
 	gtk_box_pack_start (GTK_BOX (btn_hbox), clear_btn, FALSE, TRUE, 0);
 	g_signal_connect(G_OBJECT (clear_btn), "clicked",
 			G_CALLBACK(prefs_filtering_clear_cb), NULL);
-	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), clear_btn,
-			_("Clear all the input fields in the dialog"), NULL);
+	CLAWS_SET_TIP(clear_btn,
+			_("Clear all the input fields in the dialog"));
 
 	cond_hbox = gtk_hbox_new (FALSE, 8);
 	gtk_widget_show (cond_hbox);
@@ -523,8 +520,8 @@ static void prefs_filtering_create(void)
 	gtk_box_pack_start (GTK_BOX (btn_vbox), top_btn, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT (top_btn), "clicked",
 			 G_CALLBACK(prefs_filtering_top), NULL);
-	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), top_btn,
-			_("Move the selected rule to the top"), NULL);
+	CLAWS_SET_TIP(top_btn,
+			_("Move the selected rule to the top"));
 
 #ifndef GENERIC_UMPC
 	page_up_btn = gtk_button_new_with_mnemonic (_("Page up"));
@@ -534,8 +531,8 @@ static void prefs_filtering_create(void)
 	gtk_box_pack_start (GTK_BOX (btn_vbox), page_up_btn, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT (page_up_btn), "clicked",
 			 G_CALLBACK(prefs_filtering_page_up), NULL);
-	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), page_up_btn,
-			_("Move the selected rule one page up"), NULL);
+	CLAWS_SET_TIP(page_up_btn,
+			_("Move the selected rule one page up"));
 #endif
 
 	up_btn = gtk_button_new_from_stock (GTK_STOCK_GO_UP);
@@ -543,16 +540,16 @@ static void prefs_filtering_create(void)
 	gtk_box_pack_start (GTK_BOX (btn_vbox), up_btn, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT (up_btn), "clicked",
 			 G_CALLBACK(prefs_filtering_up), NULL);
-	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), up_btn,
-			_("Move the selected rule up"), NULL);
+	CLAWS_SET_TIP(up_btn,
+			_("Move the selected rule up"));
 
 	down_btn = gtk_button_new_from_stock (GTK_STOCK_GO_DOWN);
 	gtk_widget_show (down_btn);
 	gtk_box_pack_start (GTK_BOX (btn_vbox), down_btn, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT (down_btn), "clicked",
 			 G_CALLBACK(prefs_filtering_down), NULL);
-	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), down_btn,
-			_("Move the selected rule down"), NULL);
+	CLAWS_SET_TIP(down_btn,
+			_("Move the selected rule down"));
 
 #ifndef GENERIC_UMPC
 	page_down_btn = gtk_button_new_with_mnemonic (_("Page down"));
@@ -562,8 +559,8 @@ static void prefs_filtering_create(void)
 	gtk_box_pack_start (GTK_BOX (btn_vbox), page_down_btn, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT (page_down_btn), "clicked",
 			 G_CALLBACK(prefs_filtering_page_down), NULL);
-	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), page_down_btn,
-			_("Move the selected rule one page down"), NULL);
+	CLAWS_SET_TIP(page_down_btn,
+			_("Move the selected rule one page down"));
 #endif
 
 	bottom_btn = gtk_button_new_from_stock (GTK_STOCK_GOTO_BOTTOM);
@@ -571,8 +568,8 @@ static void prefs_filtering_create(void)
 	gtk_box_pack_start (GTK_BOX (btn_vbox), bottom_btn, FALSE, FALSE, 0);
 	g_signal_connect(G_OBJECT (bottom_btn), "clicked",
 			 G_CALLBACK(prefs_filtering_bottom), NULL);
-	gtk_tooltips_set_tip(GTK_TOOLTIPS(tooltips), bottom_btn,
-			_("Move the selected rule to the bottom"), NULL);
+	CLAWS_SET_TIP(bottom_btn,
+			_("Move the selected rule to the bottom"));
 
 	if (!geometry.min_height) {
 		geometry.min_width = 500;
@@ -972,18 +969,21 @@ static gboolean prefs_filtering_search_func_cb (GtkTreeModel *model, gint column
 						GtkTreeIter *iter, gpointer search_data) 
 {
 	gchar *store_string;
-	gint key_len;
 	gboolean retval;
+	GtkTreePath *path;
 
 	gtk_tree_model_get (model, iter, column, &store_string, -1);
 
-	if (!store_string) return FALSE;
+	if (!store_string || !key) return FALSE;
 
-	key_len = strlen (key);
 
-	retval = (strncmp (key, store_string, key_len) != 0);
+	retval = (strncmp (key, store_string, strlen(key)) != 0);
 
 	g_free(store_string);
+	debug_print("selecting row\n");
+	path = gtk_tree_model_get_path(model, iter);
+	prefs_filtering_select_row(GTK_TREE_VIEW(filtering.cond_list_view), path);
+	gtk_tree_path_free(path);
 
 	return retval;
 }
@@ -1085,7 +1085,7 @@ static void prefs_filtering_condition_define_done(MatcherList * matchers)
 	}
 }
 
-static void prefs_filtering_condition_define(void)
+static void prefs_filtering_condition_define(gpointer action, gpointer data)
 {
 	gchar * cond_str;
 	MatcherList * matchers = NULL;
@@ -1121,7 +1121,7 @@ static void prefs_filtering_action_define_done(GSList * action_list)
 	}
 }
 
-static void prefs_filtering_action_define(void)
+static void prefs_filtering_action_define(gpointer action, gpointer data)
 {
 	gchar * action_str;
 	GSList * action_list = NULL;
@@ -1202,7 +1202,7 @@ fail:
 	return prop;
 }
 
-static void prefs_filtering_register_cb(void)
+static void prefs_filtering_register_cb(gpointer action, gpointer data)
 {
 	FilteringProp *prop;
 	
@@ -1217,7 +1217,7 @@ static void prefs_filtering_register_cb(void)
 	modified = TRUE;
 }
 
-static void prefs_filtering_substitute_cb(void)
+static void prefs_filtering_substitute_cb(gpointer action, gpointer data)
 {
 	gint selected_row = gtkut_list_view_get_selected_row
 		(filtering.cond_list_view);
@@ -1249,7 +1249,7 @@ static void prefs_filtering_substitute_cb(void)
 	modified = TRUE;
 }
 
-static void prefs_filtering_delete_cb(void)
+static void prefs_filtering_delete_cb(gpointer action, gpointer data)
 {
 	GtkTreeView *list_view = GTK_TREE_VIEW(filtering.cond_list_view);
 	GtkTreeModel *model;
@@ -1275,7 +1275,7 @@ static void prefs_filtering_delete_cb(void)
 	modified = TRUE;
 }
 
-static void prefs_filtering_delete_all_cb(void)
+static void prefs_filtering_delete_all_cb(gpointer action, gpointer data)
 {
 	GtkListStore *list_store;
 	
@@ -1291,12 +1291,12 @@ static void prefs_filtering_delete_all_cb(void)
 	modified = TRUE;
 }
 
-static void prefs_filtering_clear_cb(void)
+static void prefs_filtering_clear_cb(gpointer action, gpointer data)
 {
 	prefs_filtering_reset_dialog();
 }
 
-static void prefs_filtering_duplicate_cb(void)
+static void prefs_filtering_duplicate_cb(gpointer action, gpointer data)
 {
 	gint selected_row = gtkut_list_view_get_selected_row
 		(filtering.cond_list_view);	
@@ -1327,7 +1327,7 @@ static void prefs_filtering_duplicate_cb(void)
 	modified = TRUE;
 }
 
-static void prefs_filtering_top(void)
+static void prefs_filtering_top(gpointer action, gpointer data)
 {
 	gint row;
 	GtkTreeIter top, sel;
@@ -1348,7 +1348,7 @@ static void prefs_filtering_top(void)
 	modified = TRUE;
 }
 
-static void prefs_filtering_page_up(void)
+static void prefs_filtering_page_up(gpointer action, gpointer data)
 {
 	gint row, target_row, n_rows;
 	GtkTreeIter selected, target;
@@ -1380,7 +1380,7 @@ static void prefs_filtering_page_up(void)
 	gtkut_list_view_select_row(filtering.cond_list_view, target_row);
 	modified = TRUE;
 }
-static void prefs_filtering_up(void)
+static void prefs_filtering_up(gpointer action, gpointer data)
 {
 	gint row;
 	GtkTreeIter top, sel;
@@ -1401,7 +1401,7 @@ static void prefs_filtering_up(void)
 	modified = TRUE;
 }
 
-static void prefs_filtering_down(void)
+static void prefs_filtering_down(gpointer action, gpointer data)
 {
 	gint row, n_rows;
 	GtkTreeIter top, sel;
@@ -1422,7 +1422,7 @@ static void prefs_filtering_down(void)
 	modified = TRUE;
 }
 
-static void prefs_filtering_page_down(void)
+static void prefs_filtering_page_down(gpointer action, gpointer data)
 {
 	gint row, target_row, n_rows;
 	GtkTreeIter selected, target;
@@ -1455,7 +1455,7 @@ static void prefs_filtering_page_down(void)
 	modified = TRUE;
 }
 
-static void prefs_filtering_bottom(void)
+static void prefs_filtering_bottom(gpointer action, gpointer data)
 {
 	gint row, n_rows;
 	GtkTreeIter top, sel;
@@ -1508,7 +1508,7 @@ static void prefs_filtering_select_set(FilteringProp *prop)
 static gint prefs_filtering_deleted(GtkWidget *widget, GdkEventAny *event,
 				 gpointer data)
 {
-	prefs_filtering_cancel();
+	prefs_filtering_cancel(NULL, NULL);
 	return TRUE;
 }
 
@@ -1516,7 +1516,7 @@ static gboolean prefs_filtering_key_pressed(GtkWidget *widget, GdkEventKey *even
 				     gpointer data)
 {
 	if (event && event->keyval == GDK_Escape) {
-		prefs_filtering_cancel();
+		prefs_filtering_cancel(NULL, NULL);
 		return TRUE;			
 	}
 	return FALSE;
@@ -1594,7 +1594,7 @@ static gboolean prefs_filtering_check_mod(gboolean check_changed_list)
 	return FALSE;
 }
 
-static void prefs_filtering_ok(void)
+static void prefs_filtering_ok(gpointer action, gpointer data)
 {
 	if (prefs_filtering_check_mod(FALSE))
 		return;
@@ -1604,7 +1604,7 @@ static void prefs_filtering_ok(void)
 	prefs_filtering_close();
 }
 
-static void prefs_filtering_cancel(void)
+static void prefs_filtering_cancel(gpointer action, gpointer data)
 {
 	if (prefs_filtering_check_mod(TRUE))
 		return;
@@ -1745,18 +1745,19 @@ static void prefs_filtering_list_view_get_rule_info(GtkWidget *list, gint row,
 	}
 }
 
-static GtkItemFactory *prefs_filtering_popup_factory = NULL;
+static GtkActionGroup *prefs_filtering_popup_action = NULL;
 static GtkWidget *prefs_filtering_popup_menu = NULL;
 
-static GtkItemFactoryEntry prefs_filtering_popup_entries[] =
+static GtkActionEntry prefs_filtering_popup_entries[] =
 {
-	{N_("/_Delete"),		NULL, prefs_filtering_delete_cb, 0, NULL, NULL},
-	{N_("/Delete _all"),		NULL, prefs_filtering_delete_all_cb, 0, NULL, NULL},
-	{N_("/D_uplicate"),		NULL, prefs_filtering_duplicate_cb, 0, NULL, NULL},
+	{"PrefsFilteringPopup",			NULL, "PrefsFilteringPopup" },
+	{"PrefsFilteringPopup/Delete",		NULL, N_("_Delete"), NULL, NULL, G_CALLBACK(prefs_filtering_delete_cb) },
+	{"PrefsFilteringPopup/DeleteAll",	NULL, N_("Delete _all"), NULL, NULL, G_CALLBACK(prefs_filtering_delete_all_cb) },
+	{"PrefsFilteringPopup/Duplicate",	NULL, N_("D_uplicate"), NULL, NULL, G_CALLBACK(prefs_filtering_duplicate_cb) },
 #ifdef GENERIC_UMPC
-	{N_("/---"),			NULL, NULL, 0, "<Separator>", NULL},
-	{N_("/Move one page up"),		NULL, prefs_filtering_page_up, 0, NULL, NULL},
-	{N_("/Move one page down"),		NULL, prefs_filtering_page_down, 0, NULL, NULL},
+	{"PrefsFilteringPopup/---",		NULL, "---", NULL, NULL, NULL },
+	{"PrefsFilteringPopup/PageUp",		NULL, N_("Move one page up"), NULL, NULL, G_CALLBACK(prefs_filtering_page_up) },
+	{"PrefsFilteringPopup/PageDown",	NULL, N_("Move one page down"), NULL, NULL, G_CALLBACK(prefs_filtering_page_down) },
 #endif
 };
 
@@ -1784,24 +1785,32 @@ static gint prefs_filtering_list_btn_pressed(GtkWidget *widget, GdkEventButton *
 			gint row;
 
 			if (!prefs_filtering_popup_menu) {
-				gint n_entries = sizeof(prefs_filtering_popup_entries) /
-						sizeof(prefs_filtering_popup_entries[0]);
-				prefs_filtering_popup_menu = menu_create_items(prefs_filtering_popup_entries,
-								  n_entries, "<PrefsFilteringPopupMenu>",
-								  &prefs_filtering_popup_factory, list_view);
+				prefs_filtering_popup_action = cm_menu_create_action_group("PrefsFilteringPopup", prefs_filtering_popup_entries,
+					G_N_ELEMENTS(prefs_filtering_popup_entries), (gpointer)list_view);
+				MENUITEM_ADDUI("/Menus", "PrefsFilteringPopup", "PrefsFilteringPopup", GTK_UI_MANAGER_MENU)
+				MENUITEM_ADDUI("/Menus/PrefsFilteringPopup", "Delete", "PrefsFilteringPopup/Delete", GTK_UI_MANAGER_MENUITEM)
+				MENUITEM_ADDUI("/Menus/PrefsFilteringPopup", "DeleteAll", "PrefsFilteringPopup/DeleteAll", GTK_UI_MANAGER_MENUITEM)
+				MENUITEM_ADDUI("/Menus/PrefsFilteringPopup", "Duplicate", "PrefsFilteringPopup/Duplicate", GTK_UI_MANAGER_MENUITEM)
+#ifdef GENERIC_UMPC
+				MENUITEM_ADDUI("/Menus/PrefsFilteringPopup", "Separator1", "PrefsFilteringPopup/---", GTK_UI_MANAGER_SEPARATOR)
+				MENUITEM_ADDUI("/Menus/PrefsFilteringPopup", "PageUp", "PrefsFilteringPopup/PageUp", GTK_UI_MANAGER_MENUITEM)
+				MENUITEM_ADDUI("/Menus/PrefsFilteringPopup", "PageDown", "PrefsFilteringPopup/PageDown", GTK_UI_MANAGER_MENUITEM)
+#endif
+				prefs_filtering_popup_menu = gtk_menu_item_get_submenu(GTK_MENU_ITEM(
+					gtk_ui_manager_get_widget(gtkut_ui_manager(), "/Menus/PrefsFilteringPopup")) );
 			}
 
 			/* grey out some popup menu items if there is no selected row */
 			row = gtkut_list_view_get_selected_row(GTK_WIDGET(list_view));
-			menu_set_sensitive(prefs_filtering_popup_factory, "/Delete", (row > 0));
-			menu_set_sensitive(prefs_filtering_popup_factory, "/Duplicate", (row > 0));
+			cm_menu_set_sensitive("PrefsFilteringPopup/Delete", (row > 0));
+			cm_menu_set_sensitive("PrefsFilteringPopup/Duplicate", (row > 0));
 
 			/* grey out seom popup menu items if there is no row
 			   (not counting the (New) one at row 0) */
 			non_empty = gtk_tree_model_get_iter_first(model, &iter);
 			if (non_empty)
 				non_empty = gtk_tree_model_iter_next(model, &iter);
-			menu_set_sensitive(prefs_filtering_popup_factory, "/Delete all", non_empty);
+			cm_menu_set_sensitive("PrefsFilteringPopup/DeleteAll", non_empty);
 
 			gtk_menu_popup(GTK_MENU(prefs_filtering_popup_menu), 
 			    	   NULL, NULL, NULL, NULL, 
@@ -1930,7 +1939,7 @@ static void prefs_filtering_create_list_view_columns(GtkWidget *list_view)
 		 "text", PREFS_FILTERING_RULE,
 		 NULL);
 
-	gtk_tree_view_set_search_column(GTK_TREE_VIEW(list_view), 1);
+	gtk_tree_view_set_search_column(GTK_TREE_VIEW(list_view), PREFS_FILTERING_NAME);
 	gtk_tree_view_set_search_equal_func(GTK_TREE_VIEW(list_view), prefs_filtering_search_func_cb , NULL, NULL);
 	
 	gtk_tree_view_append_column(GTK_TREE_VIEW(list_view), column);		
